# Bin packing: modello

param C;				# Capacit dei bin
set Types;				# Insieme dei tipi di items
param size {Types};		# Dimensione degli items
param demand {Types};	# Numero di items per ogni tipo

#-------- MASTER --------#

param epsilon := 0.000001;

# Dati della formulazione estesa
param nCOL integer >=0 default 0;	# Numero di colonne
set COL := 1..nCOL;					# Insieme delle colonne
param a {Types,COL} integer >=0;	# N. di items di ogni tipo in ogni colonna

# Variabili
var theta {COL} integer >=0;		# N. di volte che ogni pattern  usato

# Vincoli (soddisfacimento della domanda)
subject to Demand {t in Types}:
  sum {c in COL} a[t,c] * theta[c] >= demand[t];

# Obiettivo (min n. bins)
minimize zMP: sum {c in COL} theta[c];

#-------- PRICING --------#

# Dati (variabili duali)
param lambda {Types};

# Variabili
var x {Types} integer >=0;		# N. di items di ogni tipo nel pattern

# Vincolo (capacit del bin)
subject to Capacity:
  sum {t in Types} size[t] * x[t] <= C;
  
# Obiettivo (costo ridotto)
minimize rc: 1 - sum {t in Types} lambda[t] * x[t];

#-------- Definizione problemi --------#
problem Master: theta, Demand, zMP;
problem Pricing: x, Capacity, rc;





