# Generalized assignment: modello

param nM;			# N. di macchine
param nJ;			# N. di jobs
set M := 1..nM;
set J := 1..nJ;

param a {J,M};		# Consumo di risorsa
param c {J,M}; 		# Costo di assegnamento
param b {M};		# Capacit di ogni macchina

# ------- Master ------- #

# Dati
param nCOL {M} default 0;			# N. di colonne per ogni macchina
set COL {m in M} := 1..nCOL[m];		# Set di colonne per ogni macchina
param cost {m in M,COL[m]};			# Costo di ogni colonna
param x {J,m in M,COL[m]} binary;	# Presenza di ogni job in ogni colonna
param dummycost := sum {j in J} max {m in M} (c[j,m]) + 1;	# Costo colonne fittizie

# Variabili
var theta {m in M,COL[m]} binary;	# Variabili di selezione dei pattern
var dummy {M} binary;				# Selezione colonna dummy

# Obiettivo
minimize zMP: sum {m in M, k in COL[m]} cost[m,k] * theta[m,k]
            + sum {m in M} dummycost * dummy[m];

# Vincoli
# 1. Assegnamento (duali: lambda)
subject to Assignment {j in J}:
    sum {m in M, k in COL[m]} x[j,m,k] * theta[m,k] 
  + sum {m in M} dummy[m] >= 1;

# 2. Convessit (duali: mu)
subject to Convexity {m in M}:
  sum {k in COL[m]} theta[m,k] <= 1;
   
# ------- Pricing ------- #

# Dati
param lambda {J} >=0;		# Variabili duali (premi) per ogni job
param mu {M} <=0;			# Variabili duali (penalit) per ogni macchina

# Variabili
var y {J} binary;			# Assegnamento dei job alla macchina

# Vincolo di capacit
subject to Capacity {m in M}:
  sum {j in J} a[j,m] * y[j] <= b[m];
  
# Obiettivo
minimize rc {m in M}: 
  sum {j in J} (c[j,m] - lambda[j]) * y[j] - mu[m];

# ------- Problemi ------- #

problem Master: theta, dummy, zMP, Assignment, Convexity;
problem Pricing {m in M}: y, Capacity[m], rc[m];


