# ----------------------------------------
# Cutting Stock Problem - Extended formulation
# Each column is a pattern of items to be cut/packed
# ----------------------------------------

# Data	
set TYPES;				# Set of types of items to be cut / packed
param demand {TYPES} > 0;		# N. of items for each type
param size {TYPES} > 0;			# Size of each item type
param width;				# Width of the roll / capacity of the bins

param nPAT integer >= 0, default 0;	# N. of columns
set PATTERNS := 1..nPAT;		# Set of columns

param a {TYPES,PATTERNS} integer >= 0;	# N. of items of each type in each column

# Feasibility check for each column
check {j in PATTERNS}: sum {i in TYPES} size[i] * a[i,j] <= width;

# Variables of the extended formulation
var theta {PATTERNS} integer >= 0;

# Objective
minimize zMP: sum {j in PATTERNS} theta[j];

# Constraints
subject to Demand {i in TYPES}:
   sum {j in PATTERNS} a[i,j] * theta[j] >= demand[i];

# ----------------------------------------
# Pricing subproblem: Integer Knapsack Problem
# ----------------------------------------

# Dual values
param lambda {TYPES} >= 0;

# Integer variables
var x {TYPES} integer >= 0;

# Objective function
minimize Reduced_Cost:
   1 - sum {i in TYPES} lambda[i] * x[i];

# Capacity constraint
subject to Capacity:
   sum {i in TYPES} size[i] * x[i] <= width;
